/*
 * Board initialize code for TANBAC Evaluation board TB0229.
 *
 * (C) Masami Komiya <mkomiya@sonare.it> 2004
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 */

#include <common.h>
#include <command.h>
#include <asm/addrspace.h>
#include <pci.h>
#include <flash.h>
#include <spi.h>
unsigned long mips_io_port_base = 0;

#if defined(CONFIG_PCI)
static struct pci_controller hose;

#if defined(CFG_TB0287_MINI_ITX) || defined(CFG_BLANCA)
static struct pci_device_id supported[] = {
#if defined(CFG_TB0287_MINI_ITX)
	{PCI_VENDOR_ID_CMD, PCI_DEVICE_ID_SII_680},
#elif defined(CFG_BLANCA)
	{PCI_VENDOR_ID_KURUSUGAWA_ELE, PCI_DEVICE_ID_BLANCA_IDE},
#endif
	{}
};

extern ulong ide_bus_offset[CFG_IDE_MAXBUS];
#ifdef CONFIG_IDE_PREINIT
int ide_preinit(void)
{
  pci_dev_t devno;

  if ((devno = pci_find_devices(supported, 0)) >= 0) return 0;

  // no PCI ide controller
  return 1;
}
#endif /* CONFIG_IDE_PREINIT */

#endif	/* CFG_TB0287_MINI_ITX || CFG_BLANCA */

extern void init_vr4131_pci(struct pci_controller*);

#ifdef CONFIG_SPI
#define VR4131_GIUPIODL		(volatile u16 *)(KSEG1+0x0F000144)
#define VR4131_GIUIOSELL	(volatile u16 *)(KSEG1+0x0F000140)
#define RTC_GPIO_PIN		0x40
// use GPIO6 for CE
void static rtc_set_ce(int val)
{
	u16 reg = *VR4131_GIUPIODL;
	debug( "rtc_set_ce(%d)\n", val );
	if ( val ) {
		reg |= RTC_GPIO_PIN;
	} else {
		reg &= ~(RTC_GPIO_PIN);
	}
	*VR4131_GIUPIODL = reg;
}

spi_chipsel_type spi_chipsel[1] = { rtc_set_ce };
int spi_chipsel_cnt = 1;
#endif /* CONFIG_SPI */

void pci_init_board (void)
{
#if defined(CFG_TB0287_MINI_ITX) || defined(CFG_BLANCA)
	pci_dev_t devno;
	u32 iobase;
#endif	/* CFG_TB0287_MINI_ITX || CFG_TB0287_BLANCA */

	init_vr4131_pci(&hose);
#if defined(CFG_TB0287_MINI_ITX) || defined(CFG_BLANCA)
	if ((devno = pci_find_devices(supported, 0)) >= 0) {
		pci_read_config_dword(devno, PCI_BASE_ADDRESS_0, &iobase);
		if ( iobase & PCI_BASE_ADDRESS_SPACE_IO )
			iobase = pci_io_to_phys(devno, iobase & PCI_BASE_ADDRESS_IO_MASK );
		else
			iobase = pci_mem_to_phys(devno, iobase & PCI_BASE_ADDRESS_MEM_MASK );
		printf ("IDE0 @0x%x\n", iobase);
		ide_bus_offset[0] = iobase;
#if CFG_IDE_MAXBUS > 1
		pci_read_config_dword(devno, PCI_BASE_ADDRESS_2, &iobase);
		if ( iobase & PCI_BASE_ADDRESS_SPACE_IO )
			iobase = pci_io_to_phys(devno, iobase & PCI_BASE_ADDRESS_IO_MASK );
		else
			iobase = pci_mem_to_phys(devno, iobase & PCI_BASE_ADDRESS_MEM_MASK );
		ide_bus_offset[1] = iobase;
#endif
		pci_write_config_byte(devno, PCI_LATENCY_TIMER, 0x80);

		/* this take few seconds.
		   so we do it by u-boot command, "ide reset",
		   when we realy want to access ide */
		/* ide_init(); */
	}
#endif	/* CFG_TB0287_MINI_ITX || CFG_BLANCA */
#if defined(CFG_BLANCA) && (CFG_MAX_FLASH_BANKS > 1)
	if ((devno = pci_find_device(PCI_VENDOR_ID_KURUSUGAWA_ELE,
				     PCI_DEVICE_ID_BLANCA_FLASH, 0)) >= 0) {
		extern flash_info_t flash_info[];
		extern ulong flash_get_size (ulong base, int banknum);
		pci_read_config_dword(devno, PCI_BASE_ADDRESS_0, &iobase);
		iobase = pci_mem_to_phys(devno, iobase & PCI_BASE_ADDRESS_MEM_MASK );
		printf ("Flash2: @0x%x\n", iobase);
		flash_info[1].size = flash_get_size (iobase, 1);
	}
	
#endif defined(CFG_BLANCA) && (CFG_MAX_FLASH_BANKS > 1)
}

#endif


long int initdram(int board_type)
{
	return get_ram_size ((volatile long *)CFG_SDRAM_BASE, 0x8000000);
}


int checkboard (void)
{
	printf("Board: TANBAC TB0229 ");
#if defined(CFG_TB0287_MINI_ITX)
	printf("on TB0287 ");
#elif defined(CFG_BLANCA)
	printf("on BLANCA ");
#endif
	printf("(CPU Speed %d MHz)\n", (int)CPU_CLOCK_RATE/1000000);

	return 0;
}

#ifdef CONFIG_MISC_INIT_R
int misc_init_r(void)
{
#if defined(CONFIG_SPI)
	int i;
	DECLARE_GLOBAL_DATA_PTR;

	debug ( "%s(%d)\n", __FUNCTION__ );

	/* relocate function pointer */
	for ( i = 0; i < sizeof(spi_chipsel) / sizeof(spi_chipsel[0]); i++ )
	  spi_chipsel[i] = (spi_chipsel_type)(((uchar *)spi_chipsel[i]) + gd->reloc_off);

	*VR4131_GIUIOSELL |= RTC_GPIO_PIN;
	spi_init();
#endif
	return 0;
}
#endif /* CONFIG_MISC_INIT_R */
