/*
 * (C) Copyright 2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2004
 * Mark Jonas, Freescale Semiconductor, mark.jonas@motorola.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * Modified 2006/10/25  Media Lab., Inc.  <linux@mlb.co.jp>
 *   changed for TANBC TB0287+MPC5200DIMM
 */

#include <common.h>
#include <mpc5xxx.h>
#include <pci.h>
#include <i2c.h>

#include <asm/mmu.h>
#include <asm/processor.h>


#ifndef CFG_RAMBOOT

static void sdram_start (int hi_addr)
{
	long hi_addr_bit = hi_addr ? 0x01000000 : 0;

	/* wait 200us */
	udelay(200);

	/* sdelay */
	*(vu_long *)(MPC5XXX_SDRAM + 0x90) = 0x04;

	/* unlock mode register */
	*(vu_long *)MPC5XXX_SDRAM_CTRL = SDRAM_CONTROL | 0x80000000 | hi_addr_bit;
	__asm__ volatile ("sync");

	/* precharge all banks */
	*(vu_long *)MPC5XXX_SDRAM_CTRL = SDRAM_CONTROL | 0x80000002 | hi_addr_bit;
	__asm__ volatile ("sync");

	/* set mode register: extended mode */
	*(vu_long *)MPC5XXX_SDRAM_MODE = SDRAM_EMODE;
	__asm__ volatile ("sync");

	/* set mode register: reset DLL */
	*(vu_long *)MPC5XXX_SDRAM_MODE = SDRAM_MODE | 0x04000000;
	__asm__ volatile ("sync");

	/* wait 100us */
	udelay(100);

	/* precharge all banks */
	*(vu_long *)MPC5XXX_SDRAM_CTRL = SDRAM_CONTROL | 0x80000002 | hi_addr_bit;
	__asm__ volatile ("sync");

	/* auto refresh 3 times */
	*(vu_long *)MPC5XXX_SDRAM_CTRL = SDRAM_CONTROL | 0x80000004 | hi_addr_bit;
	__asm__ volatile ("sync");
	*(vu_long *)MPC5XXX_SDRAM_CTRL = SDRAM_CONTROL | 0x80000004 | hi_addr_bit;
	__asm__ volatile ("sync");
	*(vu_long *)MPC5XXX_SDRAM_CTRL = SDRAM_CONTROL | 0x80000004 | hi_addr_bit;
	__asm__ volatile ("sync");

	/* set mode register */
	*(vu_long *)MPC5XXX_SDRAM_MODE = SDRAM_MODE;
	__asm__ volatile ("sync");

	/* normal operation  ref_en=1*/
	*(vu_long *)MPC5XXX_SDRAM_CTRL = SDRAM_CONTROL | 0x10000000 | hi_addr_bit;
	__asm__ volatile ("sync");

	/* set tap delay */
	*(vu_long *)MPC5XXX_CDM_PORCFG = 0x18000000;
	__asm__ volatile ("sync");
}
#endif

/*
 * ATTENTION: Although partially referenced initdram does NOT make real use
 *            use of CFG_SDRAM_BASE. The code does not work if CFG_SDRAM_BASE
 *            is something else than 0x00000000.
 */

long int initdram_real (int board_type)
{
#ifndef CFG_RAMBOOT
	ulong dramsize = 0;
	/* setup SDRAM chip selects */
	*(vu_long *)MPC5XXX_SDRAM_CS0CFG = ( 0x0 | 0x1A );/* 128MB at 0x0 */
	*(vu_long *)MPC5XXX_SDRAM_CS1CFG = 0;	/* disabled */
	__asm__ volatile ("sync");

	/* setup config registers */
	*(vu_long *)MPC5XXX_SDRAM_CONFIG1 = SDRAM_CONFIG1;
	*(vu_long *)MPC5XXX_SDRAM_CONFIG2 = SDRAM_CONFIG2;
	__asm__ volatile ("sync");


	/* find RAM size using SDRAM CS0 only */
//	sdram_start(0);
	sdram_start(1);

	/* scan does not work */
	dramsize = CFG_DRAM_TOTAL;
	//dramsize = get_ram_size((long *)CFG_SDRAM_BASE,0x10000000);/* Max=256M*/

	/* memory smaller than 1MB is impossible */
	if (dramsize < (1 << 20)) {
		dramsize = 0;
	}

	/* set SDRAM CS0 size according to the amount of RAM found */
	if (dramsize > 0) {
		*(vu_long *)MPC5XXX_SDRAM_CS0CFG = 
			0x13 + __builtin_ffs(dramsize >> 20) - 1;
	} else {
		*(vu_long *)MPC5XXX_SDRAM_CS0CFG = 0; /* disabled */
	}
	*(vu_long *)MPC5XXX_SDRAM_CS1CFG = 0;

	return dramsize;
	//return CFG_DRAM_TOTAL;
#endif
}


#ifdef CONFIG_BOARD_EARLY_DRAMINIT
long int initdram_early(int board_type)
{
  DECLARE_GLOBAL_DATA_PTR;
  return gd->ram_size;
};
#endif

long int initdram (int board_type)
{
#ifndef CFG_RAMBOOT

#ifdef CONFIG_BOARD_EARLY_DRAMINIT
  return initdram_early (board_type);
#else
  return initdram_real (board_type);
#endif

#else
  /* RAMBOOT just for debug, return fixed size */
  return CFG_DRAM_TOTAL;

#endif
};

#ifdef CONFIG_BOARD_EARLY_DRAMINIT
int board_early_draminit(void)
{
        DECLARE_GLOBAL_DATA_PTR;
	gd->ram_size=initdram_real(0);
#if defined(CONFIG_USE_ICACHE)
	icache_enable();
#endif
#if defined(CONFIG_INIT_MMU)
	init_mmu();
#if defined(CONFIG_USE_DCACHE)
	//dcache_enable();
#endif
#endif
	return 0;
};
#endif


int checkboard (void)
{
#if defined(CFG_TB0287_MINI_ITX)
	puts ("Board: TANBAC MPC5200DIMM (TB0286+TB0287)\n");
#elif defined(CFG_BLANCA)
	puts ("Board: TANBAC MPC5200DIMM (TB0286+BLANCA)\n");
#endif
	return 0;
}

#ifdef	CONFIG_PCI
static struct pci_controller hose;
static struct pci_device_id supported[] = {
#if defined(CFG_TB0287_MINI_ITX)
	{PCI_VENDOR_ID_CMD, PCI_DEVICE_ID_SII_680},
#elif defined(CFG_BLANCA)
	{PCI_VENDOR_ID_KURUSUGAWA_ELE, PCI_DEVICE_ID_BLANCA_IDE},
#endif
	{}
};

// this variable is defined as fixed value  in cmd_ide.c
// So, pci_init_board overwrite them by PCI config register's values
extern ulong ide_bus_offset[CFG_IDE_MAXBUS];

int ide_preinit(void)
{
  pci_dev_t devno;

  if ((devno = pci_find_devices(supported, 0)) >= 0) return 0;

  // no PCI ide controller
  return 1;
}

extern void pci_mpc5xxx_init(struct pci_controller *);

void pci_init_board(void)
{
	pci_dev_t devno;
	u32 iobase;

	pci_mpc5xxx_init(&hose);

	if ((devno = pci_find_devices(supported, 0)) >= 0) {
		pci_read_config_dword(devno, PCI_BASE_ADDRESS_0, &iobase);
		if ( iobase & PCI_BASE_ADDRESS_SPACE_IO )
			iobase = pci_io_to_phys(devno, iobase & PCI_BASE_ADDRESS_IO_MASK );
		else
			iobase = pci_mem_to_phys(devno, iobase & PCI_BASE_ADDRESS_MEM_MASK );
		printf ("IDE: @%x\n", iobase);

		ide_bus_offset[0] = iobase;
#if CFG_IDE_MAXBUS > 1
		pci_read_config_dword(devno, PCI_BASE_ADDRESS_2, &iobase);
		ide_bus_offset[1] = pci_io_to_phys(devno, iobase & PCI_BASE_ADDRESS_IO_MASK );
#endif
		/* enable mmaped I/O */
		/* pci_write_config_byte(devno, PCI_CLASS_PROG, 5 ); */
		pci_write_config_byte(devno, PCI_LATENCY_TIMER, 0x80);
		//ide_init();
	}
}

#endif

#if defined(CONFIG_OF_FLAT_TREE) && defined(CONFIG_OF_BOARD_SETUP)
#include <ft_build.h>
void
ft_board_setup(void *blob, bd_t *bd)
{
	ft_cpu_setup(blob, bd);
}
#endif
