/*
 * (C) Copyright 2007 Media Lab. Inc.,<linux@mlb.co.jp>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*****************************************************************************
 * Date & Time support for internal RTC of NEC VR41xx
 *****************************************************************************/
/*#define	DEBUG*/

#include <common.h>
#include <command.h>
#include <rtc.h>

#if defined(CONFIG_RTC_VR41XX) && (CONFIG_COMMANDS & CFG_CMD_DATE)
#include <asm/addrspace.h>
#define RTC_BASE		0x0f000100UL
#define  ETIMELREG		(RTC_BASE + 0)
#define  ETIMEMREG		(RTC_BASE + 0x02)
#define  ETIMEHREG		(RTC_BASE + 0x04)
#define rtc_readw(addr)		*(volatile u16*)(KSEG1 + addr)
#define rtc_writew(addr, data)	*(volatile u16*)(KSEG1 + addr) = data

/*****************************************************************************
 * get time
 *****************************************************************************/
void rtc_get (struct rtc_time *tmp)
{
	int i;
	ulong l[2], m[2], h[2];

	/*
	 * Elapsed Time register is 48-bit counter
	 * which count up using a 32.768 kHz.
	 * read twice and check to avoid getting a funny time
	 * when the counter is just changing
	 */
	do {
		for ( i=0; i<2; i++ ) {
			l[i] = rtc_readw(ETIMELREG);
			m[i] = rtc_readw(ETIMEMREG);
			h[i] = rtc_readw(ETIMEHREG);
		}
	} while (l[0] != l[1] || m[0] != m[1] || h[0] != h[1] );

	l[0] = (h[0] << 17) | (m[0] << 1) | (l[0] >> 15);
	to_tm(l[0], tmp);
	debug ( "Get DATE: %4d-%02d-%02d (wday=%d)  TIME: %2d:%02d:%02d\n",
		tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_wday,
		tmp->tm_hour, tmp->tm_min, tmp->tm_sec);
}

/*****************************************************************************
 * set time
 *****************************************************************************/
void rtc_set (struct rtc_time *tmp)
{
	ulong sec;
	debug ( "Set DATE: %4d-%02d-%02d (wday=%d)  TIME: %2d:%02d:%02d\n",
		tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_wday,
		tmp->tm_hour, tmp->tm_min, tmp->tm_sec);

	sec = mktime(tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_hour,
		     tmp->tm_min, tmp->tm_sec);
 
	rtc_writew(ETIMELREG, (sec << 15));
	rtc_writew(ETIMEMREG, (sec >>  1));
	rtc_writew(ETIMEHREG, (sec >> 17));
}

/*****************************************************************************
 * reset rtc
 *****************************************************************************/
void rtc_reset (void)
{
	rtc_writew(ETIMELREG, 0);
	rtc_writew(ETIMEMREG, 0);
	rtc_writew(ETIMEHREG, 0);
}

#endif	/* CONFIG_RTC_VR41XX && CFG_CMD_DATE */
